<?php

namespace Drupal\currencyapi;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface;

/**
 * Provides currency exchange rates from CurrencyAPI.com.
 */
class CurrencyApiService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The key-value expirable storage.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface
   */
  protected $keyValueExpirable;

  /**
   * Constructs a new CurrencyApiService.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    KeyValueExpirableFactoryInterface $key_value_expirable
  ) {
    $this->configFactory = $config_factory;
    $this->keyValueExpirable = $key_value_expirable;
  }

  /**
   * Fetches exchange rates from CurrencyAPI.com.
   *
   * @return array|null
   *   The exchange rates data or NULL on failure.
   */
  public function fetchExchangeRates(): ?array {
    $config = $this->configFactory->get('currencyapi.settings');
    $api_key = $config->get('api_key');
    $base_currency = $config->get('base_currency') ?? 'EUR';
    $target_currencies = $config->get('target_currencies') ?? ['USD'];

    if (empty($api_key)) {
      throw new \RuntimeException('CurrencyAPI.com API key is not configured.');
    }

    $url = $this->buildApiUrl($api_key, $base_currency, $target_currencies);
    $response = $this->makeApiRequest($url);

    if ($response && isset($response['data'])) {
      $this->storeRates($response['data']);
      return $response['data'];
    }

    return NULL;
  }

  /**
   * Builds the API URL.
   */
  protected function buildApiUrl(string $api_key, string $base_currency, array $target_currencies): string {
    return sprintf(
      'https://api.currencyapi.com/v3/latest?apikey=%s&base_currency=%s&currencies=%s',
      urlencode($api_key),
      urlencode($base_currency),
      urlencode(implode(',', $target_currencies))
    );
  }

  /**
   * Makes the API request using file_get_contents().
   */
  protected function makeApiRequest(string $url): ?array {
    $context = stream_context_create([
      'http' => [
        'timeout' => 10,
        'header' => "Accept: application/json\r\n"
      ]
    ]);

    $response = @file_get_contents($url, false, $context);

    if ($response === FALSE) {
      \Drupal::logger('currencyapi')->error('Failed to fetch exchange rates from CurrencyAPI.com');
      return NULL;
    }

    return json_decode($response, TRUE);
  }

  /**
   * Stores exchange rates in key-value storage.
   */
  protected function storeRates(array $rates): void {
    $storage = $this->keyValueExpirable->get('currencyapi_rates');
    $expiration = $this->configFactory->get('currencyapi.settings')->get('cache_expiration') ?? 86400;

    foreach ($rates as $currency => $rate_data) {
      $storage->setWithExpire($currency, $rate_data['value'], time() + $expiration);
    }
  }

  /**
   * Gets the exchange rate for a specific currency.
   */
  public function getRate(string $currency): ?float {
    $storage = $this->keyValueExpirable->get('currencyapi_rates');
    return $storage->get($currency);
  }

  /**
   * Gets all stored exchange rates.
   */
  public function getAllRates(): array {
    $storage = $this->keyValueExpirable->get('currencyapi_rates');
    return $storage->getAll();
  }

  /**
   * Converts an amount between currencies.
   */
  public function convert(float $amount, string $from, string $to): ?float {
    if ($from === $to) {
      return $amount;
    }

    $rates = $this->getAllRates();

    // Direct conversion available
    if ($from === 'EUR' && isset($rates[$to])) {
      return $amount * $rates[$to];
    }

    // Conversion through EUR
    if (isset($rates[$from]) && isset($rates[$to])) {
      $amount_in_eur = $amount / $rates[$from];
      return $amount_in_eur * $rates[$to];
    }

    return NULL;
  }

  /**
   * Gets the last update timestamp.
   */
  public function getLastUpdate(): ?int {
    $storage = $this->keyValueExpirable->get('currencyapi_rates');
    return $storage->get('_last_update');
  }
}
